// oi oi

cbuffer cbDynamic
{
	float4x4	mWorld;
	float4x4	mWorldViewProj;
	float3		watercolour;
	float		sun_shininess, sun_strength;
	float3		sun_vec;
	float4		ClipPlane;
}

Texture2D g_MeshTexture;

struct VS_INPUT
{
    float4 Pos			: POSITION;
	float3 Normal		: NORMAL;
	float2 TexCoords	: TEXCOORD0;
};

struct VS_OUTPUT
{
    float4  Pos			: SV_POSITION;    
	float	ClipDistance : SV_CLIPDISTANCE;
	float2	TexCoords	: TEXCOORD0;
	float	fade		: TEXCOORD1;
	float4	colour		: COLOR0;
};

SamplerState sDiffuse
{  
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU  = Wrap;		
	AddressV  = Wrap;		
};

VS_OUTPUT VS(VS_INPUT Input)
{
    float2 tc_transform = {-1.0f, 1.0f };

    VS_OUTPUT Output;	

	// transform position and texture, compute sun dot lighting, compute fade (deeper in water = more water color)
	Output.Pos = mul( float4( Input.Pos.xyz, 1 ), mWorldViewProj );
	Output.TexCoords = Input.TexCoords * tc_transform;
	Output.colour = saturate(dot(sun_vec,Input.Normal));
	Output.fade = 0.3 + 0.7 * saturate(-Input.Pos.y*0.04);

	Output.ClipDistance.x = dot( mul( float4( Input.Pos.xyz, 1.0f ), mWorld ), ClipPlane );

	return Output;
}

float4 PS(VS_OUTPUT Input) : SV_Target
{	
	// fade (color*texture) with watercolour
    return lerp(Input.colour*g_MeshTexture.Sample(sDiffuse, Input.TexCoords), float4(watercolour,1), Input.fade );
}

technique10 T0
{
    pass P0
    {        
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
