// oi oi

cbuffer cbConstant
{
    float		LODbias;
    float		sun_alfa, sun_theta, sun_shininess, sun_strength;
    float3		watercolour;
}

cbuffer cbDynamic
{
    float4x4	mViewProj;
    float4		view_position;
    float		g_fTime;
    float       g_FogDensity;
    bool		IsUnderwater;
}

TextureCube EnvironmentMap;
Texture2D	FresnelMap;
Texture2D	Normalmap;
Texture3D	g_FogTexture;


struct VS_INPUT
{
    float3 Pos      : POSITION;
	float3 Normal   : NORMAL;
	float2 tc		: TEXCOORD0;
};

struct VS_OUTPUT
{
    float4  Pos     : SV_POSITION;    
	float2  tc		: TEXCOORD0;
	float3  normal	: TEXCOORD1;
	float3	viewvec	: TEXCOORD2;
	float3	sun		: TEXCOORD3;
	float3  worldPos : POSITION0;
};

SamplerState sky
{  
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU  = WRAP;		
    AddressV  = WRAP;
    AddressW  = WRAP;
};

SamplerState fresnel
{  
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU  = CLAMP;		
	AddressV  = CLAMP;		
};

SamplerState nmap
{  
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU  = CLAMP;		
	AddressV  = CLAMP;		
};

SamplerState samFog
{
    Filter = ANISOTROPIC ;
    AddressU = Wrap;
    AddressV = Wrap;
    AddressW = Wrap;
};

// Gets the fog coefficient f
float4 CalcFogCoeff( float3 FogPos, float3 WorldPos ) : SV_TARGET
{
	float CameraDist;
    CameraDist = distance( view_position.xyz, WorldPos );

    // Calculate fog based on perlin noise texture (g_FogTexture)
	float4 fog = g_FogTexture.Sample(samFog, FogPos);
	
	// height effect
	fog = fog * (view_position.y-WorldPos.y-FogPos.y)*(view_position.y-WorldPos.y-FogPos.y)*0.5f + 0.001f;
	
	//effect of camera moving through fog (normalized for less variance)
    fog = fog + g_FogTexture.Sample(samFog, view_position.xyz*g_fTime*0.005f);  
    
    return exp(-(g_FogDensity*fog*CameraDist)); // f=e^(-(density*fog*distance)^2
}


float4 CalcWaterCoeff( float3 WorldPos )
{
	float CameraDist;
    CameraDist = distance( view_position.xyz, WorldPos );

	return  exp(-(0.1f*CameraDist));
}

VS_OUTPUT VS(VS_INPUT i)
{
    VS_OUTPUT   o;
    o.worldPos = i.Pos.xyz/4;

    o.Pos = mul(float4(i.Pos.xyz,1), mViewProj);    
	o.normal = normalize(i.Normal.xyz);
	o.viewvec = i.Pos.xyz - view_position.xyz/view_position.w;
	o.tc = i.tc;
	o.sun.x = cos(sun_theta)*sin(sun_alfa);
	o.sun.y = sin(sun_theta);
	o.sun.z = cos(sun_theta)*cos(sun_alfa);
	return o;
}


float4 PS(VS_OUTPUT i) : SV_Target
{
    float4 ut;

	ut.a = 1;
	float3 v = normalize(i.viewvec);
	float3 N = 2*Normalmap.Sample(nmap,i.tc)-1;
	//float3 N = float3(1, 1, 1);
	float3 R = refract(v,N,1.33);			
	R.y = - R.y;
	float f = FresnelMap.Sample(fresnel,dot(R,N));	
	float sunlight = sun_strength*pow(saturate(dot(R, i.sun)),sun_shininess);
    float3 col = EnvironmentMap.SampleBias(sky, R, LODbias) + sunlight*float3(1.2, 0.7, 0.3);	
	float3 reflcol = watercolour + saturate(30*float3(0.4,0.6,0.8)*(1-dot(float3(0,1,0),N)));
	ut.rgb = lerp(col, reflcol, f) ;

	float3 vFogDirection = { 0.0f, 0.0f, -1.0f };
	float4 vFogColor = { 0.6f, 0.6f, 0.6f, 1.0f };
	float fogF = CalcFogCoeff( i.worldPos/40.0f + (g_fTime*0.05f)*vFogDirection, i.worldPos );
	
	if( IsUnderwater )
	{
		float4 vWaterColor = { .02f, .025f, .035f, 1.0f };
		float f = CalcWaterCoeff(i.worldPos);
		return (ut*f + vWaterColor*(1-f));
	}
	
    return ut;//(ut*fogF + vFogColor*(1-fogF));WWW
}

technique10 underwater
{
    pass P0
    {      
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
