// oi oi

#define magnitude(v) sqrt(v.x*v.x+v.y*v.y+v.z*v.z)
#define falloff(m,d) (m/abs(d)-1) //hyperbolic falloff

cbuffer cbConstant
{
    float		LODbias;
    float		sun_shininess, sun_strength;
    float3		sun_vec;
    float		reflrefr_offset;
    
}

cbuffer cbDynamic
{
    float4x4	mViewProj;
    float4x4	mView;
    float4		view_position;
    float		g_fTime;
    float3		g_CameraPos;
    float       g_FogDensity;
    
    float3		g_vSpotDir;
	float3		g_vSpotPos;
	float		g_fSpotFrustum;
	float		g_fSpotIntensity;
	float4		g_vSpotDiffuse;
	float4x4	g_mSpotWorld;
}

TextureCube EnvironmentMap;
Texture3D g_FogTexture;
Texture2D FresnelMap;
Texture2D Normalmap;
Texture2D Refractionmap;
Texture2D Reflectionmap;

struct VS_INPUT
{
    float3 Pos      : POSITION;
	float3 Normal   : NORMAL;
	float2 tc		: TEXCOORD0;
};

struct VS_OUTPUT
{
    float4  Pos			: SV_POSITION;    
	float2	tc			: TEXCOORD0;
	float3  normal		: TEXCOORD1;
	float3	viewvec		: TEXCOORD2;
	float3  spotvec		: TEXCOORD3;
	float3	screenPos	: TEXCOORD4;	
	float3  fogPos		: TEXCOORD5;
	float3  worldPos	: TEXCOORD6;
	float3	screenPosDis	: TEXCOORD7;
};

SamplerState samFog
{
    Filter = ANISOTROPIC ;
    AddressU = Wrap;
    AddressV = Wrap;
    AddressW = Wrap;
};

SamplerState sky
{  
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU  = WRAP;		
    AddressV  = WRAP;
    AddressW  = WRAP;
    //SRGBTexture = true;
};

SamplerState fresnel
{  
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU  = CLAMP;		
	AddressV  = CLAMP;
};

SamplerState nmap
{  
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU  = CLAMP;		
	AddressV  = CLAMP;		
};

SamplerState refrmap
{  
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU  = CLAMP;		
	AddressV  = CLAMP;	

};

SamplerState reflmap
{  
	Filter = MIN_MAG_MIP_LINEAR;
	AddressU  = CLAMP;		
	AddressV  = CLAMP;			

};

// Gets the fog coefficient f
float4 CalcFogCoeff( float4 SVPos, float3 FogPos, float3 WorldPos ) : SV_TARGET
{
	float CameraDist;
    CameraDist = SVPos.z; //distance( g_CameraPos, WorldPos );

    // Calculate fog based on perlin noise texture (g_FogTexture)
	float4 fog = g_FogTexture.Sample(samFog, FogPos);
	
	// height effect
	fog *= (g_CameraPos.y-WorldPos.y-FogPos.y)*(g_CameraPos.y-WorldPos.y-FogPos.y)/2 + 0.001f;
	
	//effect of camera moving through fog (normalized for less variance)
    fog += g_FogTexture.Sample(samFog, g_CameraPos*g_fTime*0.005f);  
    
    return exp(-(g_FogDensity*fog*CameraDist)); // f=e^(-(density*fog*distance)^2
}

VS_OUTPUT VS(VS_INPUT i)
{
    VS_OUTPUT   o;
	o.worldPos = i.Pos.xyz;
	
    o.Pos = mul(float4(i.Pos.xyz,1), mViewProj);    
	o.normal = normalize(i.Normal.xyz);
	o.viewvec = normalize(i.Pos.xyz - view_position.xyz);
	o.spotvec = normalize(i.Pos.xyz - g_vSpotPos.xyz);
	o.tc = i.tc;	
	
	o.screenPosDis = (o.Pos.xyz / o.Pos.w);
	o.screenPosDis.xy = 0.5 + 0.5*o.screenPosDis.xy;//*float2(1,-1);
	o.screenPosDis.z = reflrefr_offset/o.screenPosDis.z;
	
	// alt screenpos

	float4 tpos = mul(float4(i.Pos.x,0,i.Pos.z,1), mViewProj);
	o.screenPos = tpos.xyz/tpos.w;
	o.screenPos.xy = 0.5 + 0.5*o.screenPos.xy*float2(1,-1);
	o.screenPos.z = reflrefr_offset/o.screenPos.z;
	
	return o;
}


float4 PS(VS_OUTPUT i) : SV_Target
{
    float4 ut;
	ut.a = 1;
	float3 v = i.viewvec;
	float3 N = 2*Normalmap.Sample(nmap,i.tc)-1;
	
	float3 R = normalize(reflect(v,N));

	float f = FresnelMap.Sample(fresnel,dot(R,N));		
	
	float3 SpotWorldDir = mul(g_vSpotDir, g_mSpotWorld);
	
	// Specular and reflection
	float4 refl = Reflectionmap.SampleBias(reflmap, i.screenPos.xy-i.screenPos.z*N.xz, LODbias);
	float3 skyrefl = EnvironmentMap.SampleBias(sky, R, LODbias);
	float3 col = lerp(skyrefl,refl.rgb,refl.a);

	// Refraction and water color
	float3 refr = Refractionmap.Sample(refrmap,i.screenPos.xy-i.screenPos.z*N.xz);
	
	// mix reflection/specular with refraction/color
	ut.rgb = lerp( refr, col, f );
	
    // spotlight
	float3 distanceVec = i.worldPos - g_vSpotPos;	
	float theta = acos( dot( normalize(distanceVec), normalize(SpotWorldDir) ) );
	if( abs(theta) < g_fSpotFrustum )
	{
	#define spot_shininess 16
		float SpotFalloff = saturate(falloff(g_fSpotFrustum,theta));
		ut.rgb += saturate(-dot( SpotWorldDir, N))*g_vSpotDiffuse*SpotFalloff*g_fSpotIntensity*0.08f;
		ut.rgb += pow(saturate(-dot( i.spotvec, R )),spot_shininess)*g_vSpotDiffuse*SpotFalloff*g_fSpotIntensity; 
	}
	
	ut.rgb += pow(saturate(-dot( sun_vec, R)), sun_shininess*0.7f)*float4(1.2f, 0.7f, 0.35f,1.0f)*5;
	
	// fog
	float3 vFogDirection = { 0.0f, 0.0f, -1.0f };
	float4 vFogColor = { 0.6f, 0.6f, 0.6f, 1.0f };
	float fogF = CalcFogCoeff( i.Pos, i.worldPos/40.0f + (g_fTime*0.05f)*vFogDirection, i.worldPos );
	
    return (ut*fogF + vFogColor*(1-fogF));
}


technique10 water
{
    pass P0
    {        
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
